'''Constants and things'''
# Copyright 2013 Christopher Foo <chris.foo@gmail.com>
# Licensed under GPLv3. See COPYING.txt for details.


NEWLINE = '\r\n'
'''String CR LF'''

NEWLINE_BYTES = b'\r\n'
'''Bytes CR LF'''

FIELD_DELIM_BYTES = NEWLINE_BYTES * 2
'''Bytes CR LF CR LF'''
