'''Document model'''
# Copyright 2013 Christopher Foo <chris.foo@gmail.com>
# Licensed under GPLv3. See COPYING.txt for details.
from .binary import *
from .block import *
from .common import *
from .field import *
from .record import *
from .warc import *
